use std::num::ParseIntError;

/// Error
#[derive(thiserror::Error, Debug)]
pub enum Error {
    /// The number of bytes is invalid
    #[error("Invalid length for {0}: {1} (expected: {2})")]
    InvalidLength(String, usize, usize),

    #[error("Invalid prefix in {0}, expecting {1}, got {2}")]
    InvalidPrefix(String, String, String),

    #[error("The string is too long (max length is {0}")]
    TooLong(usize),

    #[error("Invalid PaymentId: {0}")]
    InvalidPaymentId(#[from] ParseIntError),

    #[error("Encoding issue: {0}")]
    Bech32(#[from] bech32::DecodeError),
}
