use crate::{AccountId, Amount, PaymentId};
use serde::{Deserialize, Serialize};

#[derive(
    Clone,
    Debug,
    Eq,
    PartialEq,
    Deserialize,
    Serialize,
    borsh::BorshDeserialize,
    borsh::BorshSerialize,
)]
pub struct PaymentTo {
    /// Which account can spend this new payment
    pub to: AccountId,
    /// The amount of the payment
    #[serde(flatten)]
    pub amount: Amount,
}

/// Payment
#[derive(
    Clone,
    Debug,
    Eq,
    PartialEq,
    Deserialize,
    Serialize,
    borsh::BorshDeserialize,
    borsh::BorshSerialize,
)]
pub struct PaymentFrom {
    /// The payment ID
    pub id: PaymentId,
    /// Which account is paying
    pub from: AccountId,
    /// The amount of the payment
    #[serde(flatten)]
    pub amount: Amount,
}
