use serde::Serializer;
use std::{fmt::Display, sync::Arc};

pub(crate) fn serialize_error_to_string<S, T>(value: &T, serializer: S) -> Result<S::Ok, S::Error>
where
    S: Serializer,
    T: ToString + Display,
{
    serializer.serialize_str(&value.to_string())
}

/// Serialize an Arc<str> as a string
pub fn serialize_arc_str<S>(value: &Arc<str>, serializer: S) -> Result<S::Ok, S::Error>
where
    S: serde::Serializer,
{
    // Convert the Arc<str> to a String for serialization
    let value_str: String = value.to_string();
    serializer.serialize_str(&value_str)
}

/// Deserialize a string as an Arc<str>
pub fn deserialize_arc_str<'de, D>(deserializer: D) -> Result<Arc<str>, D::Error>
where
    D: serde::Deserializer<'de>,
{
    // Deserialize as a String and then convert it back to an Arc<str>
    let value_str: String = serde::Deserialize::deserialize(deserializer)?;
    Ok(Arc::from(value_str))
}
