//! Cursor implementation
use crate::{BaseTx, Filter, PrimaryFilter, Revision};

#[derive(Debug, PartialEq, Eq, Hash, Clone)]
/// The cursor
pub struct Cursor {
    /// The primary filter
    pub primary_filter: PrimaryFilter,
    /// The secondary filter
    pub filter: Filter,
}

impl From<Filter> for Cursor {
    fn from(filter: Filter) -> Self {
        let (primary_filter, filter) = filter.get_primary_filter();
        Self {
            primary_filter,
            filter,
        }
    }
}

impl Cursor {
    /// Check if the base transaction and the revision matches the current cursor filter
    pub fn matches(&self, base_tx: &BaseTx, revision: &Revision) -> bool {
        self.filter.matches(base_tx, revision)
    }
}
