use crate::{status, storage, Amount, Asset, RevId, Status, TxId};
use serde::Serialize;

#[derive(thiserror::Error, Debug, Serialize)]
pub enum Error {
    #[error("Payment {0} is already spent")]
    SpentPayment(usize),

    #[error("Error in status: {0}")]
    Status(#[from] status::Error),

    #[error("Payment {0} is in the incorrect status {1}")]
    InvalidPaymentStatus(usize, Status),

    #[error("Spending {0:?} and issuing {1:?}. Both amounts must match")]
    InvalidAmount(Amount, Amount),

    #[error("Missing input payment for asset {0}")]
    MissingSpendingAsset(Asset),

    #[error("Missing output payment for asset {0}")]
    MissingPaymentAsset(Asset),

    #[error("Transaction updates are not allowed")]
    TransactionUpdatesNotAllowed,

    #[error("Mismatch between spend and create payment assets")]
    MismatchSpendingPayingAssets,

    #[error("Status transition from {0} to {1} is not allowed")]
    StatusTransitionNotAllowed(Status, Status),

    #[error("Storage: {0}")]
    Storage(#[from] storage::Error),

    #[error("IO error: {0}")]
    #[serde(serialize_with = "crate::serialize_error_to_string")]
    Serialization(#[from] std::io::Error),

    #[error("No storage update when expecting")]
    /// No update was performed when expecting one
    NoUpdate,

    #[error("Invalid calculated id {0} (expected {1})")]
    InvalidTxId(TxId, TxId),

    #[error("Invalid calculated rev-id {0} (expected {1})")]
    InvalidRevId(RevId, RevId),

    #[error("Overflow")]
    Overflow,
}
